@@ -7,7 +7,7 @@ use POE::Component::IRC::Common qw(u_irc parse_mode_line parse_ban_mask);
 use POE::Component::IRC::Plugin qw(PCI_EAT_NONE);
 use base qw(POE::Component::IRC);
 
-our $VERSION = '6.08';
+our $VERSION = '6.16';
 
 # Event handlers for tracking the STATE. $self->{STATE} is used as our
 # namespace. u_irc() is used to create unique keys.
@@ -18,6 +18,7 @@ our $VERSION = '6.08';
 sub S_001 {
     my ($self) = @_;
     delete $self->{STATE};
+    delete $self->{NETSPLIT};
     $self->{STATE}->{usermode} = '';
     $self->yield(mode => $self->nick_name());
     return PCI_EAT_NONE;
@@ -70,13 +71,19 @@ sub S_join {
         $self->yield(who => $chan);
         $self->yield(mode => $chan);
         $self->yield(mode => $chan => 'b');
-        
-        if ($self->{awaypoll}) {
-            $poe_kernel->state(_away_sync => $self);
-            $poe_kernel->delay_add(_away_sync => $self->{awaypoll} => $chan);
-        }
     }
     else {
+      SWITCH: {
+        my $netsplit = "$unick!$user\@$host";
+        if ( exists $self->{NETSPLIT}->{Users}->{ $netsplit } ) {
+            # restore state from NETSPLIT if it hasn't expired.
+            my $nuser = delete $self->{NETSPLIT}->{Users}->{ $netsplit };
+            if ( ( time - $nuser->{stamp} ) < ( 60 * 60 ) ) {
+              $self->{STATE}->{Nicks}->{ $unick } = $nuser->{meta};
+              $self->_send_event(irc_nick_sync => $nick, $chan);
+              last SWITCH;
+            }
+        }
         if ( (!exists $self->{whojoiners} || $self->{whojoiners})
             && !exists $self->{STATE}->{Nicks}->{ $unick }->{Real}) {
                 $self->yield(who => $nick);
@@ -86,6 +93,7 @@ sub S_join {
             # Fake 'irc_nick_sync'
             $self->_send_event(irc_nick_sync => $nick, $chan);
         }
+      }
     }
 
     $self->{STATE}->{Nicks}->{ $unick }->{Nick} = $nick;
@@ -97,6 +105,18 @@ sub S_join {
     return PCI_EAT_NONE;
 }
 
+sub S_chan_sync {
+    my ($self, $irc) = splice @_, 0, 2;
+    my $chan = ${ $_[0] };
+
+    if ($self->{awaypoll}) {
+        $poe_kernel->state(_away_sync => $self);
+        $poe_kernel->delay_add(_away_sync => $self->{awaypoll} => $chan);
+    }
+
+    return PCI_EAT_NONE;
+}
+
 sub S_part {
     my ($self, $irc) = splice @_, 0, 2;
     my $map   = $irc->isupport('CASEMAPPING');
@@ -133,33 +153,42 @@ sub S_quit {
     my $nick  = (split /!/, ${ $_[0] })[0];
     my $msg   = ${ $_[1] };
     my $unick = u_irc($nick, $map);
+    my $netsplit = 0;
 
     push @{ $_[-1] }, [ $self->nick_channels( $nick ) ];
 
     # Check if it is a netsplit
-#    if ($msg) {
-#        SWITCH: {
-#            my @args = split /\x20/, $msg;
-#            if ( @args != 2 ) {
-#                last SWITCH;
-#           }
-#        }
-#    }
+    $netsplit = 1 if _is_netsplit( $msg );
 
     if ($unick eq u_irc($self->nick_name(), $map)) {
         delete $self->{STATE};
+        delete $self->{NETSPLIT};
     }
     else {
         for my $uchan ( keys %{ $self->{STATE}->{Nicks}->{ $unick }->{CHANS} } ) {
             delete $self->{STATE}->{Chans}->{ $uchan }->{Nicks}->{ $unick };
+            # No don't stash the channel state.
+            #$self->{NETSPLIT}->{Chans}->{ $uchan }->{NICKS}->{ $unick } = $chanstate
+            #  if $netsplit;
         }
         
-        delete $self->{STATE}->{Nicks}->{ $unick };
+        my $nickstate = delete $self->{STATE}->{Nicks}->{ $unick };
+        if ( $netsplit ) {
+          delete $nickstate->{CHANS};
+          $self->{NETSPLIT}->{Users}->{ "$unick!" . join '@', @{$nickstate}{qw(User Host)} } = 
+             { meta => $nickstate, stamp => time };
+        }
     }
 
     return PCI_EAT_NONE;
 }
 
+sub _is_netsplit {
+  my $msg = shift || return;
+  return 1 if $msg =~ /^\s*\S+\.[a-z]{2,} \S+\.[a-z]{2,}$/i;
+  return 0;
+}
+
 sub S_kick {
     my ($self, $irc) = splice @_, 0, 2;
     my $chan  = ${ $_[1] };
@@ -378,6 +407,38 @@ sub S_topic {
     return PCI_EAT_NONE;  
 }
 
+# RPL_NAMES
+sub S_353 {
+    my ($self, $irc) = splice @_, 0, 2;
+    my @data = @{ ${ $_[2] } };
+    shift @data if $data[0] =~ /^(\x40|\x2A|\x3D)$/;
+    my $chan = shift @data;
+    my @nicks = split /\s+/, shift @data;
+    my $map   = $irc->isupport('CASEMAPPING');
+    my $uchan = u_irc($chan, $map);
+    my $prefix = $irc->isupport('PREFIX') || { o => '@', v => '+' };
+    my $search = join '|', map { quotemeta } values %$prefix;
+    foreach my $user ( @nicks ) {
+       my $status;
+       if ( ($status) = $user =~ /^($search)/ ) {
+          $user =~ s/^($search)//g;
+       }
+       $status = '' unless $status;
+       my $whatever = '';
+       my $unick = u_irc($user,$map);
+       my $existing = $self->{STATE}->{Nicks}->{ $unick }->{CHANS}->{ $uchan } || '';
+       for my $mode ( keys %{ $prefix } ) {
+         if ($status =~ /\Q$prefix->{$mode}/ && $existing !~ /\Q$prefix->{$mode}/ ) {
+           $whatever .= $mode;
+         }
+       }
+       $existing .= $whatever if !$existing || $existing !~ /$whatever/;
+       $self->{STATE}->{Nicks}->{ $unick }->{CHANS}->{ $uchan } = $existing;
+       $self->{STATE}->{Chans}->{ $uchan }->{Nicks}->{ $unick } = $existing;
+    }
+    return PCI_EAT_NONE;  
+}
+
 # RPL_WHOREPLY
 sub S_352 {
     my ($self, $irc) = splice @_, 0, 2;
@@ -398,14 +459,10 @@ sub S_352 {
         $self->{STATE}->{Nicks}->{ $unick }->{IRCop} = 1 if $status =~ /\*/;
     }
     
-    if ($self->{awaypoll}) {
-        $self->{STATE}->{Nicks}->{ $unick }->{Away} = $status =~ /G/ ? 1 : 0;
-    }
-    
     if ( exists $self->{STATE}->{Chans}->{ $uchan } ) {
         my $whatever = '';
         my $existing = $self->{STATE}->{Nicks}->{ $unick }->{CHANS}->{ $uchan } || '';    
-        my $prefix = $irc->isupport('PREFIX') || { o =>, '@', v => '+' };
+        my $prefix = $irc->isupport('PREFIX') || { o => '@', v => '+' };
 
         for my $mode ( keys %{ $prefix } ) {
             if ($status =~ /\Q$prefix->{$mode}/ && $existing !~ /\Q$prefix->{$mode}/ ) {
@@ -420,12 +477,16 @@ sub S_352 {
 
         if ($self->{STATE}->{Chans}->{ $uchan }->{AWAY_SYNCH} && $unick ne u_irc($irc->nick_name(), $map)) {
             if ( $status =~ /G/ && !$self->{STATE}->{Nicks}->{ $unick }->{Away} ) {
-                $self->yield(irc_user_away => $nick, [ $self->nick_channels( $nick ) ] );
+                $self->_send_event(irc_user_away => $nick, [ $self->nick_channels( $nick ) ] );
             }
             elsif ($status =~ /H/ && $self->{STATE}->{Nicks}->{ $unick }->{Away} ) {
-                $self->yield(irc_user_back => $nick, [ $self->nick_channels( $nick ) ] );
+                $self->_send_event(irc_user_back => $nick, [ $self->nick_channels( $nick ) ] );
             }
         }
+
+        if ($self->{awaypoll}) {
+            $self->{STATE}->{Nicks}->{ $unick }->{Away} = $status =~ /G/ ? 1 : 0;
+        }
     }
     
     return PCI_EAT_NONE;
@@ -1120,13 +1181,16 @@ POE::Component::IRC::State's constructors, and its C<connect> event, all
 take the same arguments as L<POE::Component::IRC|POE::Component::IRC> does, as
 well as two additional ones:
 
-'AwayPoll', the interval (in seconds) in which to poll (i.e. C<WHO #channel>)
+B<'AwayPoll'>, the interval (in seconds) in which to poll (i.e. C<WHO #channel>)
 the away status of channel members. Defaults to 0 (disabled). If enabled, you
 will receive C<irc_away_sync_*> / L<C<irc_user_away>|/"irc_user_away"> /
 L<C<irc_user_back>|/"irc_user_back"> events, and will be able to use the
-L<C<is_away>|/"is_away"> method for users other than yourself.
+L<C<is_away>|/"is_away"> method for users other than yourself. This can cause
+a lot of increase in traffic, especially if you are on big channels, so if you
+do use this, you probably don't want to set it too low. For reference, X-Chat
+uses 300 seconds (5 minutes).
 
-'WhoJoiners', a boolean indicating whether the component should send a
+B<'WhoJoiners'>, a boolean indicating whether the component should send a
 C<WHO nick> for every person which joins a channel. Defaults to on
 (the C<WHO> is sent). If you turn this off, L<C<is_operator>|/"is_operator">
 will not work and L<C<nick_info>|/"nick_info"> will only return the keys
@@ -1319,12 +1383,12 @@ Takes no parameters. Returns the current user mode set for the bot.
 
 =head2 Augmented events
 
-New paramater are added to the following L<POE::Component::IRC|POE::Component::IRC>
-events.
+New paramaters are added to the following
+L<POE::Component::IRC|POE::Component::IRC> events.
 
 =head3 C<irc_quit>
 
-See also L<C<irc_quit>|POE::Component::IRC/"irc_quit">> in
+See also L<C<irc_quit>|POE::Component::IRC/"irc_quit"> in
 L<POE::Component::IRC|POE::Component::IRC>.
 
 Additional paramater C<ARG2> contains an arrayref of channel names that are
@@ -1353,7 +1417,7 @@ individual.
 =head3 C<irc_socketerr>
 
 These three all have two additional parameters. C<ARG1> is a hash of
-information about your IRC user (see L<C<nick_info>|/"nick_info">, while
+information about your IRC user (see L<C<nick_info>|/"nick_info">), while
 C<ARG2> is a hash of the channels you were on (see 
 L<C<channels>|/"channels">).
 
@@ -1412,6 +1476,9 @@ C<ARG1> is an arrayref of channel names that are common to the nickname
 and the component. You will only receive this event if you specified a value
 for B<'AwayPoll'> in L<C<spawn>|POE::Component::IRC/"spawn">.
 
+B<Note:> This above is only for users I<other than yourself>. To know when you
+change your own away status, register for the C<irc_305> and C<irc_306> events.
+
 =head3 C<irc_user_back>
 
 Sent when an IRC user unsets his/her away status. C<ARG0> is the nickname,
@@ -1419,9 +1486,8 @@ C<ARG1> is an arrayref of channel names that are common to the nickname and
 the component. You will only receive this event if you specified a value for
 B<'AwayPoll'> in L<C<spawn>|POE::Component::IRC/"spawn">.
 
-The following two C<irc_*> events are the same as their
-L<POE::Component::IRC|POE::Component::IRC> counterparts, with the additional
-parameters:
+B<Note:> This above is only for users I<other than yourself>. To know when you
+change your own away status, register for the C<irc_305> and C<irc_306> events.
 
 =head3 C<irc_user_mode>
 
@@ -1434,7 +1500,7 @@ The component gathers information by registering for C<irc_quit>, C<irc_nick>,
 C<irc_join>, C<irc_part>, C<irc_mode>, C<irc_kick> and various numeric replies.
 When the component is asked to join a channel, when it joins it will issue
 'WHO #channel', 'MODE #channel', and 'MODE #channel b'. These will solicit
-between them the numerics, >C<irc_352>, C<irc_324> and C<irc_329>, respectively.
+between them the numerics, C<irc_352>, C<irc_324> and C<irc_329>, respectively.
 When someone joins a channel the bot is on, it issues a 'WHO nick'. You may
 want to ignore these. 
 
